﻿local addon, addonData = ...
local arrowAvailable, arrowTextAvailable = false, false;

if not DRTSOptions then
    DRTSOptions = {
        enabled = true,
        debug = false,
        arrowEnabled = true,
        ignoretm = false,
        hidden = { },
    }
end

local function GetDistance(x, y)
    local startX, startY = UnitPosition("player")
    local dX = startX - x
    local dY = startY - y
    return (dX * dX + dY * dY) ^ 0.5
end

local function SelectQuest(treasure, type)
    if treasure.flags == 131073 then
        if type == "map" then
            return treasure.quests[1]
        elseif type == "complete" then
            return treasure.quests[2]
        else
            --print("Unknown type "..type.. " for flags 131073")
            return 0
        end
    elseif treasure.flags == 65541 then
        if type == "map" then
            return treasure.quests[2]
        elseif type == "complete" then
            return treasure.quests[1]
        else
            --print("Unknown type "..type.. " for flags 65541")
            return 0
        end
    elseif treasure.flags == 65537 then
        if type == "map" then
            return treasure.quests[2]
        elseif type == "complete" then
            return treasure.quests[1]
        else
            --print("Unknown type "..type.. " for flags 65537")
            return 0
        end
    elseif treasure.flags == 524310 then
        if type == "choise1" then
            return treasure.quests[1]
        elseif type == "choise2" then
            return treasure.quests[2]
        elseif type == "map" then
            return treasure.quests[3]
        elseif type == "complete" then
            return treasure.quests[4]
        else
            --print("Unknown type "..type.. " for flags 524310")
            return 0
        end
    elseif treasure.flags == 0 then
        --print("Treasure "..treasure.id.." has 0 flags")
        return 0
    else
        --print("Unknown flags "..treasure.flags)
        return 0
    end
end

local function GetTreasureCompletion(treasure)
    local trquest, tmquest, c1quest, c2quest = SelectQuest(treasure, "complete"), SelectQuest(treasure, "map"), SelectQuest(treasure, "choise1"), SelectQuest(treasure, "choise2")
    local trCheck = trquest == 0 or not IsQuestFlaggedCompleted(trquest)
    local tmCheck = DRTSOptions.ignoretm or tmquest == 0 or not IsQuestFlaggedCompleted(tmquest)
    local choise1Check = c1quest == 0 or IsQuestFlaggedCompleted(c1quest)
    local choise2Check = c2quest == 0 or IsQuestFlaggedCompleted(c2quest)
    return trCheck, tmCheck, choise1Check, choise2Check, trquest, tmquest, c1quest, c2quest
end

local function GetClosestTreasure()
    local dist = 99999999
    local closest = nil
    for k,v in pairs(addonData.treasures) do
        local dist2 = GetDistance(v.x, v.y)
        local trCheck, tmCheck, choise1Check, choise2Check = GetTreasureCompletion(v)
        if dist2 < dist and (DRTSOptions.debug or not DRTSOptions.hidden[k]) and (DRTSOptions.debug or (trCheck and tmCheck and choise1Check and choise2Check)) then
            dist = dist2
            closest = v
        end
    end
    return closest, dist
end

if DBM then
    arrowAvailable = true
    if DBMArrow then
        local text = DBMArrow:CreateFontString(nil, "OVERLAY", "GameTooltipText")
        text:SetWidth(150)
        text:SetHeight(60)
        text:SetPoint("CENTER", DBMArrow, "CENTER", 0, -35)
        text:SetTextColor(1, 1, 1, 1)
        text:SetJustifyH("CENTER")
        DBMArrow.text = text
        arrowTextAvailable = true
    end
end

local function ShowArrow(x, y, text)
    if arrowAvailable then
        DBM.Arrow:ShowRunTo(x,y,nil,5)
    end
    if arrowTextAvailable then
        if not DBMArrow.text:IsShown() then
            DBMArrow.text:Show()
        end
        DBMArrow.text:SetText(text)
    end
end

local function HideArrow()
    if arrowAvailable then
        DBM.Arrow:Hide()
    end
    if arrowTextAvailable then
        DBMArrow.text:Hide()
    end
end

local interval = 1

local TreasureMapsIds = {
    [1116] = true,
    [1158] = true,
}

local TreasureQuestsDesc = {
    [36466] = "Talador",
    [36467] = "Spires of Arak",
    [36465] = "Gorgrond",
    [34557] = "Frostfire",
    [36468] = "Nagrand",
    [36464] = "Shadowmoon Valley",
    [36249] = "requires Lumber Mill",
    [36250] = "requires Lumber Mill",
    [36251] = "requires Sparring Arena",
    [36252] = "requires Sparring Arena",
}

local currentmapid = -1;

local function onUpdate(self,elapsed)
    if not DRTSOptions.enabled then return end
    if not DRTSOptions.arrowEnabled then return end
    currentmapid = select(4, UnitPosition("player"))
    if not TreasureMapsIds[currentmapid] then return end
    interval = interval + elapsed
    if interval >= 1 then
        interval = 0
        local closest, dist = GetClosestTreasure()
        if not closest then return end
        if DRTSOptions.debug then
            ShowArrow(closest.x,closest.y, format("%s (%u)\n%.02f yd", closest.name, closest.id, dist))
        else
            ShowArrow(closest.x,closest.y, format("%s\n%.02f yd", closest.name, dist))
        end
    end
end

local MinimapRadius = {
    Inside = {
        [0] = 150.0,
        [1] = 120.0,
        [2] = 90.0,
        [3] = 60.0,
        [4] = 40.0,
        [5] = 25.0,
    },
    Outside = {
        [0] = 233.333324,
        [1] = 199.999992,
        [2] = 166.66666,
        [3] = 133.333328,
        [4] = 99.999996,
        [5] = 66.666664,
    }
}

local function onEvent(self,event,...)
    if event == "MINIMAP_UPDATE_ZOOM" then
        -- update minimap zoom scale
        local Minimap = Minimap;
        local curZoom = Minimap:GetZoom();
        if ( GetCVar("minimapZoom") == GetCVar("minimapInsideZoom") ) then
            if ( curZoom < 2 ) then
                Minimap:SetZoom(curZoom + 1);
            else
                Minimap:SetZoom(curZoom - 1);
            end
        end
        if ( GetCVar("minimapZoom")+0 == Minimap:GetZoom() ) then
            self.minimapOutside = true;
        else
            self.minimapOutside = false;
        end
        Minimap:SetZoom(curZoom);
    elseif event == "ADDON_LOADED" then
        local addonName = ...
        if addonName ~= addon then return end
        if not DRTSOptions.hidden then
            DRTSOptions.hidden = { }
        end
    end
end

local frame = CreateFrame("Frame")
if DRTSOptions.enabled then
    frame:SetScript("OnUpdate", onUpdate)
else
    frame:SetScript("OnUpdate", nil)
end
frame:SetScript("OnEvent", onEvent)
frame:RegisterEvent("MINIMAP_UPDATE_ZOOM");
frame:RegisterEvent("ADDON_LOADED");

SLASH_DRTS1 = "/drts";

SlashCmdList["DRTS"] = function(msg, editBox)
    if msg:lower() == "on" then
        frame:SetScript("OnUpdate", onUpdate)
        DRTSOptions.enabled = true
        DEFAULT_CHAT_FRAME:AddMessage("DraenorTreasures enabled")
    elseif msg:lower() == "off" then
        frame:SetScript("OnUpdate", nil)
        DRTSOptions.enabled = false
        HideArrow()
        DEFAULT_CHAT_FRAME:AddMessage("DraenorTreasures disabled")
    elseif msg:lower() == "arrow" then
        if not arrowAvailable then
            DEFAULT_CHAT_FRAME:AddMessage("DraenorTreasures arrow not available. Please install DBM in order to use arrow.")
            return
        end
        DRTSOptions.arrowEnabled = not DRTSOptions.arrowEnabled
        DEFAULT_CHAT_FRAME:AddMessage("DraenorTreasures arrow "..(DRTSOptions.arrowEnabled and "enabled" or "disabled"))
        if not DRTSOptions.arrowEnabled then
            HideArrow()
        end
    elseif msg:lower() == "debug" then
        DRTSOptions.debug = not DRTSOptions.debug
        DEFAULT_CHAT_FRAME:AddMessage("DraenorTreasures debug "..(DRTSOptions.debug and "enabled" or "disabled"))
    elseif msg:lower() == "closest" then
        local closest, dist = GetClosestTreasure()
        DEFAULT_CHAT_FRAME:AddMessage("DraenorTreasures: "..closest.id.." ("..closest.name..")")
    elseif msg:lower() == "minimap" then
        local x, y = UnitPosition("player")
        local zoom = Minimap:GetZoom()
        local radius = frame.minimapOutside and MinimapRadius.Outside[zoom] or MinimapRadius.Inside[zoom]
        local h = (y + radius) - (y - radius)
        local w = (x + radius) - (x - radius)
        --print(format("h: %f, w: %f", h, w))
    elseif msg:lower() == "hidepoi" then
        --DRTSOptions.hidden[i] = true
        --DEFAULT_CHAT_FRAME:AddMessage("DraenorTreasures: debug "..(DRTSOptions.debug and "enabled" or "disabled"))
    elseif msg:lower() == "itm" then
        DRTSOptions.ignoretm = not DRTSOptions.ignoretm
        DEFAULT_CHAT_FRAME:AddMessage("DraenorTreasures: treasure map quests now "..(DRTSOptions.ignoretm and "ignored" or "used"))
    else
        DEFAULT_CHAT_FRAME:AddMessage("Usage: /drts on|off")
        DEFAULT_CHAT_FRAME:AddMessage("Usage: /drts arrow (toggle)")
        DEFAULT_CHAT_FRAME:AddMessage("Usage: /drts debug (toggle)")
    end
end

local function WorldPosToScreenPos(x, y)
    local isMapDungeon = true
    local floorIndex, minY, minX, maxY, maxX = GetCurrentMapDungeonLevel();
    local zoneIndex, locLeft, locTop, locRight, locBottom;

    if not (minX and minY and maxX and maxY) then
        isMapDungeon = false
        zoneIndex, locLeft, locTop, locRight, locBottom = GetCurrentMapZone()
        if not (locLeft and locTop and locRight and locBottom) then
            return 0, 0, false;
        end
    end

    if isMapDungeon then
        local h, w = abs(maxY - minY), abs(maxX - minX);
        local sx = (maxX - x) / w;
        local sy = (maxY - y) / h;
        local onMap = y < maxY and y > minY and x < maxX and x > minX;
        return sy, sx, onMap;
    else
        local h, w = abs(locRight - locLeft), abs(locBottom - locTop);
        local sx = (locTop - x) / w;
        local sy = (locLeft - y) / h;
        local onMap = y < locLeft and y > locRight and x < locTop and x > locBottom;
        return sy, sx, onMap;
    end
end

local function TreasureMap_ResetPOI(button)
    button:SetWidth(32);
    button:SetHeight(32);
    button.Texture:SetWidth(16);
    button.Texture:SetHeight(16);
    button.Texture:SetPoint("CENTER", 0, 0);
    button.Texture:SetTexture("Interface\\Minimap\\POIIcons");

    button.specialPOIInfo = nil;
end

local debugMsgFormat = "Treasure %s (%u):\nxpos %f\nypos %f\ntracking quest %u (%s)\ntreasuremap quest %u (%s)\ndistance %.02f";

local function TreasureMapPOI_OnClick(self, button)
    if not DRTSOptions.debug or button == "RightButton" then
        WorldMapButton_OnClick(WorldMapButton, button);
        return;
    end
    local t = addonData.treasures[self.id];
    local trCheck, tmCheck, choise1Check, choise2Check, trquest, tmquest, c1quest, c2quest = GetTreasureCompletion(t);
    local trstate = trquest ~= 0 and (trCheck and "Not completed" or "Completed") or "No quest";
    local tmstate = tmquest ~= 0 and (tmCheck and "Not completed" or "Completed") or "No quest";
    print(format(debugMsgFormat, t.name, t.id, t.x, t.y, trquest, trstate, tmquest, tmstate, GetDistance(t.x, t.y)));
end

local function TreasureMap_CreatePOI(index)
    local button = CreateFrame("Button", "TreasureMapFramePOI"..index, WorldMapButton);
    button:RegisterForClicks("LeftButtonUp", "RightButtonUp");
    button:SetScript("OnEnter", WorldMapPOI_OnEnter);
    button:SetScript("OnLeave", WorldMapPOI_OnLeave);
    button:SetScript("OnClick", TreasureMapPOI_OnClick);

    button.Texture = button:CreateTexture(button:GetName().."Texture", "BACKGROUND");

    TreasureMap_ResetPOI(button);
    return button;
end

local function DrawTreasurePOI()
    local worldMapID, isContinent = GetCurrentMapAreaID();
    local mapID = GetAreaMapInfo(worldMapID);
    local showTreasures = (not isContinent) and (worldMapID > 0) and DRTSOptions.enabled and (TreasureMapsIds[currentmapid] or TreasureMapsIds[mapID]);

    for k,v in pairs(addonData.treasures) do
        local treasureMapPOI = _G["TreasureMapFramePOI"..k] or TreasureMap_CreatePOI(k);

        local trCheck, tmCheck, choise1Check, choise2Check, _, tmquest, c1quest, _ = GetTreasureCompletion(v)

        if showTreasures and (DRTSOptions.debug or (not DRTSOptions.hidden[k] and trCheck and tmCheck and choise1Check and choise2Check)) then
            local x, y, onMap = WorldPosToScreenPos(v.x, v.y);
            if onMap then
                local name;
                if DRTSOptions.debug then
                    name = format("%s (%u, %.02f, %.02f)", v.name, v.id, x * 100, y * 100);
                else
                    name = format("%s (%.02f, %.02f)", v.name, x * 100, y * 100);
                end

                local description = v.desc ~= "" and v.desc or format("Draenor Treasure: %s!", TreasureQuestsDesc[c1quest] or TreasureQuestsDesc[tmquest] or "Unknown");

                x = x * WorldMapButton:GetWidth();
                y = -y * WorldMapButton:GetHeight();
                treasureMapPOI:SetPoint("CENTER", "WorldMapButton", "TOPLEFT", x, y);

                TreasureMap_ResetPOI(treasureMapPOI);

                local x1, x2, y1, y2 = GetPOITextureCoords(197);

                treasureMapPOI.Texture:SetTexCoord(x1, x2, y1, y2);

                treasureMapPOI.id = k;
                treasureMapPOI.name = name;
                treasureMapPOI.description = description;
                treasureMapPOI.mapLinkID = nil;

                treasureMapPOI:Show();
            else
                treasureMapPOI:Hide();
            end
        else
            treasureMapPOI:Hide();
        end
    end
end

local function MyWorldMapFrame_OnUpdate(self, e)
    WorldMapFrame_OnUpdate(self, e);
    DrawTreasurePOI();
end

WorldMapFrame:SetScript("OnUpdate", MyWorldMapFrame_OnUpdate);
